/* 
** Bridge support functions
**
** @file brsupport.c
** 
** -----------------------------------------------------------------------------
** Enduro/X Middleware Platform for Distributed Transaction Processing
** Copyright (C) 2015, ATR Baltic, SIA. All Rights Reserved.
** This software is released under one of the following licenses:
** GPL or ATR Baltic's license for commercial use.
** -----------------------------------------------------------------------------
** GPL license:
** 
** This program is free software; you can redistribute it and/or modify it under
** the terms of the GNU General Public License as published by the Free Software
** Foundation; either version 2 of the License, or (at your option) any later
** version.
**
** This program is distributed in the hope that it will be useful, but WITHOUT ANY
** WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
** PARTICULAR PURPOSE. See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along with
** this program; if not, write to the Free Software Foundation, Inc., 59 Temple
** Place, Suite 330, Boston, MA 02111-1307 USA
**
** -----------------------------------------------------------------------------
** A commercial use license is available from ATR Baltic, SIA
** contact@atrbaltic.com
** -----------------------------------------------------------------------------
*/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <sys_mqueue.h>
#include <errno.h>
#include <time.h>
#include <sys/param.h>


#include <atmi.h>
#include <ndrstandard.h>
#include <ndebug.h>
#include <atmi_int.h>
#include <atmi_shm.h>

#include <ndrxdcmn.h>
#include "gencall.h"
/*---------------------------Externs------------------------------------*/
/*---------------------------Macros-------------------------------------*/
/*---------------------------Enums--------------------------------------*/
/*---------------------------Typedefs-----------------------------------*/
/*---------------------------Globals------------------------------------*/
/*---------------------------Statics------------------------------------*/
/*---------------------------Prototypes---------------------------------*/

/**
 * Dump node stack
 * @param stack
 */
public void br_dump_nodestack(char *stack, char *msg)
{
    int i=0;
    int nodeid;
    char node_stack_str[CONF_NDRX_NODEID_COUNT*4];
    node_stack_str[0] = EOS;
    char tmp[10];
    int len = strlen(stack);
    
    for (i=0; i<len; i++)
    {
        nodeid = stack[i];
        if (i+1!=len)
            sprintf(tmp, "%d->", nodeid);
        else
            sprintf(tmp, "%d", nodeid);
        strcat(node_stack_str, tmp);
    }
    NDRX_LOG(log_error, "%s: [%s]", msg, node_stack_str);
    
}

/**
 * Fill reply queue, either local (if no nodes in stack), or another birdge service.
 * @param call
 * @return 
 */
public int fill_reply_queue(char *nodestack, 
            char *org_reply_to, char *reply_to)
{
    int i;
    int len;
    int nodeid;
    int ret=SUCCEED;
    
    *reply_to = EOS;
    /* So we are going to do reply, we should scan the reply stack
     * chose the bridge, and reduce the callstack
     */
    br_dump_nodestack(nodestack, "Node stack before bridge select");
    
    if ((len = strlen(nodestack)) > 0 )
    {
        NDRX_LOG(log_debug, "Selecting bridge for reply");
        for (i=0; i<len; i++)
        {
            nodeid = nodestack[i];
            if (ndrx_shm_bridge_is_connected(nodeid))
            {
                /* get the bridge service first... */
#ifndef EX_USE_EPOLL
                int is_bridge;
                char tmpsvc[MAXTIDENT+1];
                
                sprintf(tmpsvc, NDRX_SVC_BRIDGE, nodeid);
                
                if (SUCCEED!=ndrx_shm_get_svc(tmpsvc, reply_to, &is_bridge))
                {
                    NDRX_LOG(log_error, "Failed to get bridge svc: [%s]", 
                            tmpsvc);
                    FAIL_OUT(ret);
                }
#else
                /* epoll mode direct call: */
                sprintf(reply_to, NDRX_SVC_QBRDIGE, G_atmi_conf.q_prefix, nodeid);
#endif
                
                nodestack[i] = EOS;
                br_dump_nodestack(nodestack, "Node stack after bridge select");
                break;
            }
        }
        
        if (EOS==*reply_to)
        {
            ret=FAIL;
            NDRX_LOG(log_error, "No bridge to send reply to!");
        }
    }
    /* If have something in node stack */
    else
    {
        /* this was local call, so reply on q */
        strcpy(reply_to, org_reply_to);
    }
    
out:
    return ret;
}
